package com.thomaso.bridge.impossible;

/* Copyright (@)1999, Thomas Andrews */
/* Free for non-commercial use       */
import java.math.BigInteger;
import com.thomaso.numeric.BigCombo;
import com.thomaso.numeric.IncreasingSequence;

/**
 * Implements an encoding of all bridge deals.
 * 
 * This is the encoding used by the <a href="http://www.best.com/~thomaso/decode/">original Impossible BridgeBook.</a>
 *
 * @author Thomas Andrews
 */
public class AndrewsBook extends ImpossibleBook {
    private final static BigInteger Choose52_13=BigCombo.choose(52,13);
    private final static BigInteger Choose39_13=BigCombo.choose(39,13);
    private final static BigInteger Choose26_13=BigCombo.choose(26,13);

    public int[] getPage(BigInteger index) {
        BigInteger div[]=index.divideAndRemainder(Choose52_13);
        int seqN[]=IncreasingSequence.decode(div[1],13);
        index=div[0];
        div=index.divideAndRemainder(Choose39_13);
        int seqE[]=IncreasingSequence.decode(div[1],13);
        index=div[0];
        div=index.divideAndRemainder(Choose26_13);
        int seqS[]=IncreasingSequence.decode(div[1],13);
	return decode(seqN,seqE,seqS);
    }

    private static int NORTH=0;
    private static int EAST=1;
    private static int SOUTH=2;
    private static int WEST=3;
    protected int[] decode(int seqN[],int seqE[],int seqS[]) {
	int result[]=new int[52];
	int countN=0;
	int indexN=0;
	int countE=0;
	int indexE=0;
	int countS=0;
	int indexS=0;

	for (int i=0; i<52; i++) {
	    indexN++;
	    if (countN<13 && seqN[countN]==indexN-1) {
	        result[i]=NORTH;
		countN++;
		continue;
	    }

	    indexE++;
            if (countE<13 && seqE[countE]==indexE-1) {
		countE++;
	        result[i]=EAST;
		continue;
	    }

	    indexS++;
            if (countS<13 && seqS[countS]==indexS-1) {
		countS++;
	        result[i]=SOUTH;
		continue;
	    }

	    result[i]=WEST;
	}
	return result;
    }

    public BigInteger index(int whom[]) {
	int sequences[][]=new int[3][13];
	int i;
        int j;
        for (j=0; j<3; j++) {
	    int count=0;
	    int loc=0;
	    for (i=0; i<52; i++) {	
		if (j==whom[i]) {
		    sequences[j][loc++]=count;
	        }
		if (j<=whom[i]) {
			count++;
	        }
	    }
        }

        BigInteger seqS=IncreasingSequence.encode(sequences[2]);
        BigInteger seqE=IncreasingSequence.encode(sequences[1]);
        BigInteger seqN=IncreasingSequence.encode(sequences[0]);
	BigInteger result=seqS.multiply(Choose39_13).
		add(seqE).multiply(Choose52_13).
		add(seqN);
	
	return result;
    }

    public static void main(String args[]) {
      ImpossibleBook.main(new AndrewsBook(),args);
    }
}
