package com.thomaso.bridge.impossible;
/* Copyright (@)1999, Thomas Andrews */
/* Free for non-commercial use       */
import java.math.BigInteger;
import java.util.Vector;

/**
 * An abstract class which, when extended, indexes all possible bridge
 * hands.
 *
 * The main reason to make this an abstract class, rather than an
 * interface, is to allow the definition of getPageAsVector and to record
 * the 'Pages' value.
 *
 * @author Thomas Andrews
 */
public abstract class ImpossibleBook {

  /**
   * The number of bridge deals.
   */
  public final static 
    BigInteger Pages=new BigInteger("53644737765488792839237440000");

  /**
   * Convert an integer between 0 and Pages-1 to a deal.
   *
   * The deal is represented as an array of 52 integers between 0 and 3.
   * The ith element determines who gets the ith card from the deck.
   * This means that this doesn't really determine a deal - you also need
   * a set of hand and card encodings - North=0,East=1,South=2,West=3,
   * AceofSpades=51,...,TwoOfClubs=0 is one such encoding.
   * 
   * @param pageNo The page requested.
   * @returns An array of 52 integers from 0 to 3.
   */
  public abstract int[] getPage(BigInteger pageNo);

  /**
   * Abstract: Should take a deal and find its index in the book
   * 
   * @param whom An array telling where each cards is placed.
   *
   * @returns A page number in the book.
   */
  public abstract BigInteger index(int whom[]);

  /**
   * Calls encode then wraps the result as a Vector of Integers.
   *
   * @param pageNo The page number in the book.
   * @returns A Vector of 52 Integer objects from 0 to 3.
   */
  public Vector getPageAsVector(BigInteger pageNo) {
    int arr[]=getPage(pageNo);
    Vector vector=new Vector();
    for (int i=0; i<arr.length; i++) {
      vector.addElement(new Integer(arr[i]));
    }
    return vector;
  }

  /**
   * Get a page number from a string.
   *
   * @param pageString A string containing a page number.
   * @returns An array of 52 integers between 0 and 3.
   */
  public int[] getPage(String pageString) {
    BigInteger pageno=new BigInteger(pageString);
    return getPage(pageno);
  }

  /**
   * Run a test for a given book.  Each argument is read as an
   * index into the given book, and the page is decoded then
   * encoded again. 
   *
   * @param book An implemention of this class
   * @param args A sequence of page numbers to test.
   */
  public static void main(ImpossibleBook book,String args[]) {
    int i=0;
    while (i<args.length) {
      BigInteger index=new BigInteger(args[i++]);
      System.out.println(book.getPageAsVector(index));
      int whom[]=book.getPage(index);
      BigInteger recode=book.index(whom);
      System.out.println("Re-encoded: " + recode);
    }   
  }   
}
