package com.thomaso.bridge.impossible;
/* Copyright (@)1999, Thomas Andrews */
/* Free for non-commercial use       */
import java.math.BigInteger;

/**
 * An encoding from Richard Pavlicek.
 *
 * This encoding comes from an email discussion with Richard Pavlicek.
 * It is considerably simpler and more natural than my encoding.  See his
 * <a href="http://www.gate.net/~pavlicek/mapping.htm">Mapping of Bridge Deals</a>
 * page for a description of the algorithm.
 * 
 */
public class PavlicekBook extends ImpossibleBook {

  /**
   * Utility function returns val*numer/denom.
   */
  static private BigInteger fraction(BigInteger val,long numer,long denom)
  {
    return val.multiply(BigInteger.valueOf(numer))
      .divide(BigInteger.valueOf(denom));
  }

  public int[] getPage(BigInteger pageNo) {
    int result[]=new int[52];
    int C=52;
    int cardsNeeded[]={13, 13, 13, 13};
    BigInteger K=ImpossibleBook.Pages;

    for (C=52; C>0; C--) {
      for (int hand=0; hand<4; hand++) {
	BigInteger X=fraction(K,cardsNeeded[hand],C);
	if (pageNo.compareTo(X)<0) {
	  result[C-1]=hand;
	  K=X;
	  cardsNeeded[hand]--;
	  break;
	} else {
	  pageNo=pageNo.subtract(X);
	}
      }
    }
    return result;
  }
    
  public BigInteger index(int whom[]) {
    BigInteger minimum=BigInteger.valueOf(0);
    BigInteger width=ImpossibleBook.Pages;

    int cardsNeeded[]={13,13,13,13};

    int C;

    for (C=52; C>0; C--) {
      int hand=0;
      int skipped=0;
      int goesTo=whom[C-1];
      while (hand<goesTo) { skipped = skipped + cardsNeeded[hand]; hand++;}
      minimum=
	minimum.add(fraction(width,skipped,C));
      width=fraction(width,cardsNeeded[goesTo],C);
      cardsNeeded[goesTo]--;
    }
    return minimum;
  }

  public static void main(String args[]) {
    ImpossibleBook.main(new PavlicekBook(),args);
  }
}
