package com.thomaso.numeric;
/* Copyright (@)1999, Thomas Andrews */
/* Free for non-commercial use       */
import java.util.Vector;
import java.math.BigInteger;

/**
 *  Utility class for implementing the Squashed Order.
 *
 *  Used by the Impossible Bridge Book.
 *
 */
public class IncreasingSequence {

  /**
   * Do a lookup in the squashed order.
   *
   */
  static public int[] decode(BigInteger index,int length) {
    int result[]=new int[length];
    while (length>0) {
	BigInteger lastValue=new BigInteger("0");
	int num=length;
	BigInteger nextValue;
	while (index.compareTo(nextValue=BigCombo.choose(num,length))>=0) {
	  num++;
	  lastValue=nextValue;
	}
	result[length-1]=num-1;
	index=index.subtract(lastValue);
	length--;
    }
    return result;
  }

  /**
   * Same as decode, only computs a Vector.
   */
  static public Vector decodeToVector(BigInteger index, int length) {
	int arr[]=decode(index,length);
        Vector result=new Vector();
	for (int i=0; i<arr.length; i++) {
	   result.addElement(new Integer(arr[i]));
	}
	return result;
  }

  /**
   * Starting with an increasing sequence of natural numbers,
   * determine the index of that number in the squashed order.
   */
  static public BigInteger encode(int sequence[]) {
    BigInteger sum=new BigInteger("0");
    for (int i=0; i<sequence.length; i++) {
      sum=sum.add(BigCombo.choose(sequence[i],i+1));
    }
    return sum;
  }

  static public void main(String args[]) {
    System.out.println(decodeToVector(new BigInteger("15"),3));
  }
}
